/* global describe, it */
'use strict';

var assert = require('power-assert');
var async = require('../../');

describe('#whilst', function() {

  it('should execute until test is false', function(done) {

    var count = 0;
    var limit = 5;
    var order = {
      test: [],
      iterator: []
    };
    var test = function() {
      order.test.push(count);
      return count < limit;
    };
    var iterator = function(callback) {
      order.iterator.push(count++);
      callback();
    };

    async.whilst(test, iterator, function(err) {
      if (err) {
        return done(err);
      }

      assert.deepEqual(order.iterator, [0, 1, 2, 3, 4]);
      assert.deepEqual(order.test, [0, 1, 2, 3, 4, 5]);
      done();
    });
  });

  it('should execute with binding until test is false', function(done) {

    var count = 0;
    var limit = 5;
    var order = {
      test: [],
      iterator: []
    };
    var result = [];
    var test = function() {
      order.test.push(count);
      return count < limit;
    };
    var iterator = function(callback) {
      result.push(this.pow(count, 2));
      order.iterator.push(count++);
      callback();
    };

    async.whilst(test, iterator, function(err) {
      if (err) {
        return done(err);
      }

      assert.deepEqual(order.iterator, [0, 1, 2, 3, 4]);
      assert.deepEqual(order.test, [0, 1, 2, 3, 4, 5]);
      assert.deepEqual(result, [0, 1, 4, 9, 16]);
      done();
    }, Math);
  });
});

describe('#doWhilst', function() {

  it('should execute until test is false', function(done) {

    var count = 0;
    var limit = 5;
    var order = {
      test: [],
      iterator: []
    };
    var test = function() {
      order.test.push(count);
      return count < limit;
    };
    var iterator = function(callback) {
      order.iterator.push(count++);
      callback();
    };

    async.doWhilst(iterator, test, function(err) {
      if (err) {
        return done(err);
      }

      assert.deepEqual(order.iterator, [0, 1, 2, 3, 4]);
      assert.deepEqual(order.test, [1, 2, 3, 4, 5]);
      done();
    });
  });

  it('should execute with binding until test is false', function(done) {

    var count = 0;
    var limit = 5;
    var order = {
      test: [],
      iterator: []
    };
    var result = [];
    var test = function() {
      order.test.push(count);
      return count < limit;
    };
    var iterator = function(callback) {
      result.push(this.pow(count, 2));
      order.iterator.push(count++);
      callback();
    };

    async.doWhilst(iterator, test, function(err) {
      if (err) {
        return done(err);
      }

      assert.deepEqual(order.iterator, [0, 1, 2, 3, 4]);
      assert.deepEqual(order.test, [1, 2, 3, 4, 5]);
      assert.deepEqual(result, [0, 1, 4, 9, 16]);
      done();
    }, Math);
  });

  it('should execute until test is false and apply params', function(done) {

    var count = 0;
    var limit = 5;
    var order = {
      test: [],
      iterator: []
    };
    var test = function(c) {
      order.test.push(c);
      return c < limit;
    };
    var iterator = function(callback) {
      order.iterator.push(count++);
      callback(null, count);
    };

    async.doWhilst(iterator, test, function(err) {
      if (err) {
        return done(err);
      }

      assert.deepEqual(order.iterator, [0, 1, 2, 3, 4]);
      assert.deepEqual(order.test, [1, 2, 3, 4, 5]);
      done();
    });

  });

});

